@@ -1,5 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-1998, 2000-2009 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2010, 2012-2013, 2015-2016 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _MESSAGE_H
 #define _MESSAGE_H
@@ -47,6 +47,7 @@ enum format_type
   format_objc,
   format_sh,
   format_python,
+  format_python_brace,
   format_lisp,
   format_elisp,
   format_librep,
@@ -66,9 +67,12 @@ enum format_type
   format_qt,
   format_qt_plural,
   format_kde,
-  format_boost
+  format_kde_kuit,
+  format_boost,
+  format_lua,
+  format_javascript
 };
-#define NFORMATS 24     /* Number of format_type enum values.  */
+#define NFORMATS 28     /* Number of format_type enum values.  */
 extern DLL_VARIABLE const char *const format_language[NFORMATS];
 extern DLL_VARIABLE const char *const format_language_pretty[NFORMATS];
 
@@ -111,6 +115,30 @@ enum is_wrap
 #endif
 
 
+/* Kinds of syntax checks which apply to strings.  */
+enum syntax_check_type
+{
+  sc_ellipsis_unicode,
+  sc_space_ellipsis,
+  sc_quote_unicode,
+  sc_bullet_unicode
+};
+#define NSYNTAXCHECKS 4
+extern DLL_VARIABLE const char *const syntax_check_name[NSYNTAXCHECKS];
+
+/* Is current msgid subject to a syntax check?  */
+#if 0
+enum is_syntax_check
+{
+  undecided,
+  yes,
+  no
+};
+#else /* HACK - C's enum concept is so stupid */
+#define is_syntax_check is_format
+#endif
+
+
 struct altstr
 {
   const char *msgstr;
@@ -172,6 +200,9 @@ struct message_ty
   /* Do we want the string to be wrapped in the emitted PO file?  */
   enum is_wrap do_wrap;
 
+  /* Do we want to apply extra syntax checks on the string?  */
+  enum is_syntax_check do_syntax_check[NSYNTAXCHECKS];
+
   /* The prev_msgctxt, prev_msgid and prev_msgid_plural strings appearing
      before the message, if present.  Generated by msgmerge.  */
   const char *prev_msgctxt;
