@@ -1,5 +1,6 @@
 /* Converts Uniforum style .po files to binary .mo files
-   Copyright (C) 1995-1998, 2000-2007, 2009-2010, 2012 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2007, 2009-2010, 2012, 2014-2016, 2018-2019 Free Software
+   Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -49,6 +50,7 @@
 #include "write-tcl.h"
 #include "write-qt.h"
 #include "write-desktop.h"
+#include "write-xml.h"
 #include "propername.h"
 #include "message.h"
 #include "open-catalog.h"
@@ -61,10 +63,15 @@
 #include "msgl-check.h"
 #include "msgl-iconv.h"
 #include "concat-filename.h"
+#include "its.h"
+#include "locating-rule.h"
+#include "search-path.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
 
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
 /* Contains exit status for case in which no premature exit occurs.  */
 static int exit_status;
 
@@ -110,6 +117,14 @@ static const char *desktop_base_directory;
 static hash_table desktop_keywords;
 static bool desktop_default_keywords = true;
 
+/* XML mode output file specification.  */
+static bool xml_mode;
+static const char *xml_locale_name;
+static const char *xml_template_name;
+static const char *xml_base_directory;
+static const char *xml_language;
+static its_rule_list_ty *xml_its_rules;
+
 /* We may have more than one input file.  Domains with same names in
    different files have to merged.  So we need a list of tables for
    each output file.  */
@@ -180,6 +195,7 @@ static const struct option long_options[] =
   { "java", no_argument, NULL, 'j' },
   { "java2", no_argument, NULL, CHAR_MAX + 5 },
   { "keyword", required_argument, NULL, 'k' },
+  { "language", required_argument, NULL, 'L' },
   { "locale", required_argument, NULL, 'l' },
   { "no-hash", no_argument, NULL, CHAR_MAX + 6 },
   { "output-file", required_argument, NULL, 'o' },
@@ -196,6 +212,7 @@ static const struct option long_options[] =
   { "use-untranslated", no_argument, NULL, CHAR_MAX + 12 },
   { "verbose", no_argument, NULL, 'v' },
   { "version", no_argument, NULL, 'V' },
+  { "xml", no_argument, NULL, 'x' },
   { NULL, 0, NULL, 0 }
 };
 
@@ -211,11 +228,14 @@ static struct msg_domain *new_domain (const char *name, const char *file_name);
 static bool is_nonobsolete (const message_ty *mp);
 static void read_catalog_file_msgfmt (char *filename,
                                       catalog_input_format_ty input_syntax);
-static string_list_ty *get_languages (const char *directory);
 static int msgfmt_desktop_bulk (const char *directory,
                                 const char *template_file_name,
                                 hash_table *keywords,
                                 const char *file_name);
+static int msgfmt_xml_bulk (const char *directory,
+                            const char *template_file_name,
+                            its_rule_list_ty *its_rules,
+                            const char *file_name);
 
 
 int
@@ -232,6 +252,7 @@ main (int argc, char *argv[])
 
   /* Set default value for global variables.  */
   alignment = DEFAULT_OUTPUT_ALIGNMENT;
+  byteswap = 0 ^ ENDIANNESS;
 
   /* Set program name for messages.  */
   set_program_name (argv[0]);
@@ -239,10 +260,8 @@ main (int argc, char *argv[])
   error_one_per_line = 1;
   exit_status = EXIT_SUCCESS;
 
-#ifdef HAVE_SETLOCALE
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
-#endif
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
@@ -252,8 +271,8 @@ main (int argc, char *argv[])
   /* Ensure that write errors on stdout are detected.  */
   atexit (close_stdout);
 
-  while ((opt = getopt_long (argc, argv, "a:cCd:D:fhjl:o:Pr:vV", long_options,
-                             NULL))
+  while ((opt = getopt_long (argc, argv, "a:cCd:D:fhjl:L:o:Pr:vVx",
+                             long_options, NULL))
          != EOF)
     switch (opt)
       {
@@ -281,6 +300,7 @@ main (int argc, char *argv[])
         csharp_base_directory = optarg;
         tcl_base_directory = optarg;
         desktop_base_directory = optarg;
+        xml_base_directory = optarg;
         break;
       case 'D':
         dir_list_append (optarg);
@@ -313,6 +333,10 @@ main (int argc, char *argv[])
         csharp_locale_name = optarg;
         tcl_locale_name = optarg;
         desktop_locale_name = optarg;
+        xml_locale_name = optarg;
+        break;
+      case 'L':
+        xml_language = optarg;
         break;
       case 'o':
         output_file_name = optarg;
@@ -333,6 +357,9 @@ main (int argc, char *argv[])
       case 'V':
         do_version = true;
         break;
+      case 'x':
+        xml_mode = true;
+        break;
       case CHAR_MAX + 1: /* --check-accelerators */
         check_accelerators = true;
         if (optarg != NULL)
@@ -402,6 +429,7 @@ main (int argc, char *argv[])
         break;
       case CHAR_MAX + 16: /* --template=TEMPLATE */
         desktop_template_name = optarg;
+        xml_template_name = optarg;
         break;
       default:
         usage (EXIT_FAILURE);
@@ -414,11 +442,11 @@ main (int argc, char *argv[])
       printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "1995-1998, 2000-2010");
+              "1995-2019");
       printf (_("Written by %s.\n"), proper_name ("Ulrich Drepper"));
       exit (EXIT_SUCCESS);
     }
@@ -428,16 +456,20 @@ There is NO WARRANTY, to the extent permitted by law.\n\
     usage (EXIT_SUCCESS);
 
   /* Test whether we have a .po file name as argument.  */
-  if (optind >= argc && !(desktop_mode && desktop_base_directory))
+  if (optind >= argc
+      && !(desktop_mode && desktop_base_directory)
+      && !(xml_mode && xml_base_directory))
     {
       error (EXIT_SUCCESS, 0, _("no input file given"));
       usage (EXIT_FAILURE);
     }
-  if (optind < argc && desktop_mode && desktop_base_directory)
+  if (optind < argc
+      && ((desktop_mode && desktop_base_directory)
+          || (xml_mode && xml_base_directory)))
     {
       error (EXIT_SUCCESS, 0,
              _("no input file should be given if %s and %s are specified"),
-             "--desktop", "-d");
+             desktop_mode ? "--desktop" : "--xml", "-d");
       usage (EXIT_FAILURE);
     }
 
@@ -449,10 +481,11 @@ There is NO WARRANTY, to the extent permitted by law.\n\
       | (csharp_resources_mode ? 4 : 0)
       | (tcl_mode ? 8 : 0)
       | (qt_mode ? 16 : 0)
-      | (desktop_mode ? 32 : 0);
+      | (desktop_mode ? 32 : 0)
+      | (xml_mode ? 64 : 0);
     static const char *mode_options[] =
       { "--java", "--csharp", "--csharp-resources", "--tcl", "--qt",
-        "--desktop" };
+        "--desktop", "--xml" };
     /* More than one bit set?  */
     if (modes & (modes - 1))
       {
@@ -558,6 +591,34 @@ There is NO WARRANTY, to the extent permitted by law.\n\
           usage (EXIT_FAILURE);
         }
     }
+  else if (xml_mode)
+    {
+      if (xml_template_name == NULL)
+        {
+          error (EXIT_SUCCESS, 0,
+                 _("%s requires a \"--template template\" specification"),
+                 "--xml");
+          usage (EXIT_FAILURE);
+        }
+      if (output_file_name == NULL)
+        {
+          error (EXIT_SUCCESS, 0,
+                 _("%s requires a \"-o file\" specification"),
+                 "--xml");
+          usage (EXIT_FAILURE);
+        }
+      if (xml_base_directory != NULL && xml_locale_name != NULL)
+        error (EXIT_FAILURE, 0,
+               _("%s and %s are mutually exclusive in %s"),
+               "-d", "-l", "--xml");
+      if (xml_base_directory == NULL && xml_locale_name == NULL)
+        {
+          error (EXIT_SUCCESS, 0,
+                 _("%s requires a \"-l locale\" specification"),
+                 "--xml");
+          usage (EXIT_FAILURE);
+        }
+    }
   else
     {
       if (java_resource_name != NULL)
@@ -600,6 +661,68 @@ There is NO WARRANTY, to the extent permitted by law.\n\
       exit (exit_status);
     }
 
+  if (xml_mode)
+    {
+      char **its_dirs;
+      char **dirs;
+      locating_rule_list_ty *its_locating_rules;
+      const char *its_basename;
+
+      its_dirs = get_search_path ("its");
+      its_locating_rules = locating_rule_list_alloc ();
+      for (dirs = its_dirs; *dirs != NULL; dirs++)
+        locating_rule_list_add_from_directory (its_locating_rules, *dirs);
+
+      its_basename = locating_rule_list_locate (its_locating_rules,
+                                                xml_template_name,
+                                                xml_language);
+
+      if (its_basename != NULL)
+        {
+          size_t j;
+
+          xml_its_rules = its_rule_list_alloc ();
+          for (j = 0; its_dirs[j] != NULL; j++)
+            {
+              char *its_filename =
+                xconcatenated_filename (its_dirs[j], its_basename, NULL);
+              struct stat statbuf;
+              bool ok = false;
+
+              if (stat (its_filename, &statbuf) == 0)
+                ok = its_rule_list_add_from_file (xml_its_rules, its_filename);
+              free (its_filename);
+              if (ok)
+                break;
+            }
+          if (its_dirs[j] == NULL)
+            {
+              its_rule_list_free (xml_its_rules);
+              xml_its_rules = NULL;
+            }
+        }
+      locating_rule_list_free (its_locating_rules);
+
+      for (dirs = its_dirs; *dirs != NULL; dirs++)
+        free (*dirs);
+      free (its_dirs);
+
+      if (xml_its_rules == NULL)
+        error (EXIT_FAILURE, 0, _("cannot locate ITS rules for %s"),
+               xml_template_name);
+    }
+
+  /* Bulk processing mode for XML files.
+     Process all .po files in xml_base_directory.  */
+  if (xml_mode && xml_base_directory)
+    {
+      exit_status = msgfmt_xml_bulk (xml_base_directory,
+                                     xml_template_name,
+                                     xml_its_rules,
+                                     output_file_name);
+      exit (exit_status);
+    }
+
   /* The -o option determines the name of the domain and therefore
      the output file.  */
   if (output_file_name != NULL)
@@ -705,6 +828,15 @@ There is NO WARRANTY, to the extent permitted by law.\n\
           if (desktop_keywords.table != NULL)
             hash_destroy (&desktop_keywords);
         }
+      else if (xml_mode)
+        {
+          if (msgdomain_write_xml (domain->mlp, canon_encoding,
+                                   xml_locale_name,
+                                   xml_template_name,
+                                   xml_its_rules,
+                                   domain->file_name))
+            exit_status = EXIT_FAILURE;
+        }
       else
         {
           if (msgdomain_write_mo (domain->mlp, domain->domain_name,
@@ -810,6 +942,8 @@ Operation mode:\n"));
       --qt                    Qt mode: generate a Qt .qm file\n"));
       printf (_("\
       --desktop               Desktop Entry mode: generate a .desktop file\n"));
+      printf (_("\
+      --xml                   XML mode: generate XML file\n"));
       printf ("\n");
       printf (_("\
 Output file location:\n"));
@@ -873,6 +1007,22 @@ Desktop Entry mode options:\n"));
   -k, --keyword               do not to use default keywords\n"));
       printf (_("\
 The -l, -o, and --template options are mandatory.  If -D is specified, input\n\
+files are read from the directory instead of the command line arguments.\n"));
+      printf ("\n");
+      printf (_("\
+XML mode options:\n"));
+      printf (_("\
+  -l, --locale=LOCALE         locale name, either language or language_COUNTRY\n"));
+      printf (_("\
+  -L, --language=NAME         recognise the specified XML language\n"));
+      printf (_("\
+  -o, --output-file=FILE      write output to specified file\n"));
+      printf (_("\
+  --template=TEMPLATE         an XML file used as a template\n"));
+      printf (_("\
+  -d DIRECTORY                base directory of .po files\n"));
+      printf (_("\
+The -l, -o, and --template options are mandatory.  If -D is specified, input\n\
 files are read from the directory instead of the command line arguments.\n"));
       printf ("\n");
       printf (_("\
@@ -1001,7 +1151,6 @@ struct msgfmt_catalog_reader_ty
   DEFAULT_CATALOG_READER_TY
 
   bool has_header_entry;
-  bool has_nonfuzzy_header_entry;
 };
 
 
@@ -1015,7 +1164,6 @@ msgfmt_constructor (abstract_catalog_reader_ty *that)
   default_constructor (that);
 
   this->has_header_entry = false;
-  this->has_nonfuzzy_header_entry = false;
 }
 
 
@@ -1034,23 +1182,9 @@ msgfmt_parse_debrief (abstract_catalog_reader_ty *that)
       if (!this->has_header_entry)
         {
           multiline_error (xasprintf ("%s: ", this->file_name),
-                           xasprintf (_("\
-warning: PO file header missing or invalid\n")));
+                           xasprintf (_("warning: PO file header missing or invalid\n")));
           multiline_error (NULL,
-                           xasprintf (_("\
-warning: charset conversion will not work\n")));
-        }
-      else if (!this->has_nonfuzzy_header_entry)
-        {
-          /* Has only a fuzzy header entry.  Since the versions 0.10.xx
-             ignore a fuzzy header entry and even give an error on it, we
-             give a warning, to increase operability with these older
-             msgfmt versions.  This warning can go away in January 2003.  */
-          multiline_warning (xasprintf ("%s: ", this->file_name),
-                             xasprintf (_("warning: PO file header fuzzy\n")));
-          multiline_warning (NULL,
-                             xasprintf (_("\
-warning: older versions of msgfmt will give an error on this\n")));
+                           xasprintf (_("warning: charset conversion will not work\n")));
         }
     }
 }
@@ -1063,7 +1197,7 @@ msgfmt_set_domain (default_catalog_reader_ty *this, char *name)
   /* If no output file was given, we change it with each 'domain'
      directive.  */
   if (!java_mode && !csharp_mode && !csharp_resources_mode && !tcl_mode
-      && !qt_mode && !desktop_mode && output_file_name == NULL)
+      && !qt_mode && !desktop_mode && !xml_mode && output_file_name == NULL)
     {
       size_t correct;
 
@@ -1073,13 +1207,14 @@ msgfmt_set_domain (default_catalog_reader_ty *this, char *name)
           exit_status = EXIT_FAILURE;
           if (correct == 0)
             {
-              error (0, 0, _("\
-domain name \"%s\" not suitable as file name"), name);
+              error (0, 0,
+                     _("domain name \"%s\" not suitable as file name"), name);
               return;
             }
           else
-            error (0, 0, _("\
-domain name \"%s\" not suitable as file name: will use prefix"), name);
+            error (0, 0,
+                   _("domain name \"%s\" not suitable as file name: will use prefix"),
+                   name);
           name[correct] = '\0';
         }
 
@@ -1171,8 +1306,6 @@ msgfmt_frob_new_message (default_catalog_reader_ty *that, message_ty *mp,
           if (is_header (mp))
             {
               this->has_header_entry = true;
-              if (!mp->is_fuzzy)
-                this->has_nonfuzzy_header_entry = true;
             }
           else
             /* We don't count the header entry in the statistic so place
@@ -1202,8 +1335,8 @@ msgfmt_comment_special (abstract_catalog_reader_ty *that, const char *s)
       if (!include_fuzzies && check_compatibility && !warned)
         {
           warned = true;
-          error (0, 0, _("\
-%s: warning: source file contains fuzzy translation"),
+          error (0, 0,
+                 _("%s: warning: source file contains fuzzy translation"),
                  gram_pos.file_name);
         }
     }
@@ -1297,35 +1430,30 @@ add_languages (string_list_ty *languages, string_list_ty *desired_languages,
 
 /* Compute the languages list by reading the "LINGUAS" envvar or the
    LINGUAS file under DIRECTORY.  */
-static string_list_ty *
-get_languages (const char *directory)
+static void
+get_languages (string_list_ty *languages, const char *directory)
 {
   char *envval;
-  string_list_ty *languages;
-  string_list_ty *desired_languages = NULL;
-  char *linguas_file_name;
+  string_list_ty real_desired_languages, *desired_languages = NULL;
+  char *linguas_file_name = NULL;
   struct stat statbuf;
   FILE *fp;
   size_t line_len = 0;
   char *line_buf = NULL;
 
-  languages = string_list_alloc ();
   envval = getenv ("LINGUAS");
   if (envval)
     {
-      desired_languages = string_list_alloc ();
-      add_languages (desired_languages, NULL, envval, strlen (envval));
+      string_list_init (&real_desired_languages);
+      add_languages (&real_desired_languages, NULL, envval, strlen (envval));
+      desired_languages = &real_desired_languages;
     }
 
   linguas_file_name = xconcatenated_filename (directory, "LINGUAS", NULL);
   if (stat (linguas_file_name, &statbuf) < 0)
     {
       error (EXIT_SUCCESS, 0, _("%s does not exist"), linguas_file_name);
-      string_list_free (languages);
-      if (desired_languages != NULL)
-        string_list_free (desired_languages);
-      free (linguas_file_name);
-      return NULL;
+      goto out;
     }
 
   fp = fopen (linguas_file_name, "r");
@@ -1333,11 +1461,7 @@ get_languages (const char *directory)
     {
       error (EXIT_SUCCESS, 0, _("%s exists but cannot read"),
              linguas_file_name);
-      string_list_free (languages);
-      if (desired_languages != NULL)
-        string_list_free (desired_languages);
-      free (linguas_file_name);
-      return NULL;
+      goto out;
     }
 
   while (!feof (fp))
@@ -1367,11 +1491,132 @@ get_languages (const char *directory)
 
   free (line_buf);
   fclose (fp);
-  if (desired_languages != NULL)
-    string_list_free (desired_languages);
-  free (linguas_file_name);
 
-  return languages;
+ out:
+  if (desired_languages != NULL)
+    string_list_destroy (desired_languages);
+  free (linguas_file_name);
+}
+
+static void
+msgfmt_operand_list_init (msgfmt_operand_list_ty *operands)
+{
+  operands->items = NULL;
+  operands->nitems = 0;
+  operands->nitems_max = 0;
+}
+
+static void
+msgfmt_operand_list_destroy (msgfmt_operand_list_ty *operands)
+{
+  size_t i;
+
+  for (i = 0; i < operands->nitems; i++)
+    {
+      free (operands->items[i].language);
+      message_list_free (operands->items[i].mlp, 0);
+    }
+  free (operands->items);
+}
+
+static void
+msgfmt_operand_list_append (msgfmt_operand_list_ty *operands,
+                            const char *language,
+                            message_list_ty *messages)
+{
+  msgfmt_operand_ty *operand;
+
+  if (operands->nitems == operands->nitems_max)
+    {
+      operands->nitems_max = operands->nitems_max * 2 + 1;
+      operands->items = xrealloc (operands->items,
+                                  sizeof (msgfmt_operand_ty)
+                                  * operands->nitems_max);
+    }
+
+  operand = &operands->items[operands->nitems++];
+  operand->language = xstrdup (language);
+  operand->mlp = messages;
+}
+
+static int
+msgfmt_operand_list_add_from_directory (msgfmt_operand_list_ty *operands,
+                                        const char *directory)
+{
+  string_list_ty languages;
+  void *saved_dir_list;
+  int retval = 0;
+  size_t i;
+
+  string_list_init (&languages);
+  get_languages (&languages, directory);
+
+  if (languages.nitems == 0)
+    return 0;
+
+  /* Reset the directory search list so only .po files under DIRECTORY
+     will be read.  */
+  saved_dir_list = dir_list_save_reset ();
+  dir_list_append (directory);
+
+  /* Read all .po files.  */
+  for (i = 0; i < languages.nitems; i++)
+    {
+      const char *language = languages.item[i];
+      message_list_ty *mlp;
+      char *input_file_name;
+      int nerrors;
+
+      current_domain = new_domain (MESSAGE_DOMAIN_DEFAULT,
+                                   add_mo_suffix (MESSAGE_DOMAIN_DEFAULT));
+
+      input_file_name = xconcatenated_filename ("", language, ".po");
+      read_catalog_file_msgfmt (input_file_name, &input_format_po);
+      free (input_file_name);
+
+      /* The domain directive is not supported in the bulk execution mode.
+         Thus, domain_list should always contain a single domain.  */
+      assert (current_domain == domain_list && domain_list->next == NULL);
+      mlp = current_domain->mlp;
+      free (current_domain);
+      current_domain = domain_list = NULL;
+
+      /* Remove obsolete messages.  They were only needed for duplicate
+         checking.  */
+      message_list_remove_if_not (mlp, is_nonobsolete);
+
+      /* Perform all kinds of checks: plural expressions, format
+         strings, ...  */
+      nerrors =
+        check_message_list (mlp,
+                            /* Untranslated and fuzzy messages have already
+                               been dealt with during parsing, see below in
+                               msgfmt_frob_new_message.  */
+                            0, 0,
+                            1, check_format_strings, check_header,
+                            check_compatibility,
+                            check_accelerators, accelerator_char);
+
+      retval += nerrors;
+      if (nerrors > 0)
+        {
+          error (0, 0,
+                 ngettext ("found %d fatal error", "found %d fatal errors",
+                           nerrors),
+                 nerrors);
+          continue;
+        }
+
+      /* Convert the messages to Unicode.  */
+      iconv_message_list (mlp, NULL, po_charset_utf8, NULL);
+
+      msgfmt_operand_list_append (operands, language, mlp);
+    }
+
+  string_list_destroy (&languages);
+  dir_list_restore (saved_dir_list);
+
+  return retval;
 }
 
 /* Helper function to support 'bulk' operation mode of --desktop.
@@ -1384,93 +1629,60 @@ msgfmt_desktop_bulk (const char *directory,
                      hash_table *keywords,
                      const char *file_name)
 {
-  string_list_ty *languages = NULL;
-  message_list_ty **messages = NULL;
-  void *saved_dir_list;
-  int retval = 0;
-  size_t i;
+  msgfmt_operand_list_ty operands;
+  int nerrors, status;
 
-  languages = get_languages (directory);
-  if (!languages)
-    {
-      retval = EXIT_FAILURE;
-      goto out;
-    }
-
-  /* Reset the directory search list so only .po files under DIRECTORY
-     will be read.  */
-  saved_dir_list = dir_list_save_reset ();
-  dir_list_append (directory);
+  msgfmt_operand_list_init (&operands);
 
   /* Read all .po files.  */
-  messages = XNMALLOC (languages->nitems, message_list_ty *);
-  for (i = 0; i < languages->nitems; i++)
+  nerrors = msgfmt_operand_list_add_from_directory (&operands, directory);
+  if (nerrors > 0)
     {
-      const char *language = languages->item[i];
-      char *input_file_name;
-      int nerrors;
-
-      current_domain = new_domain (file_name, file_name);
-
-      input_file_name = xconcatenated_filename ("", language, ".po");
-      read_catalog_file_msgfmt (input_file_name, &input_format_po);
-      free (input_file_name);
-
-      /* The domain directive is not supported by --desktop mode.
-         Thus, domain_list should always contain a single domain.  */
-      assert (current_domain == domain_list && domain_list->next == NULL);
-      messages[i] = current_domain->mlp;
-      free (current_domain);
-      current_domain = domain_list = NULL;
-
-      /* Remove obsolete messages.  They were only needed for duplicate
-         checking.  */
-      message_list_remove_if_not (messages[i], is_nonobsolete);
-
-      /* Perform all kinds of checks: plural expressions, format
-         strings, ...  */
-      nerrors =
-        check_message_list (messages[i],
-                            /* Untranslated and fuzzy messages have already
-                               been dealt with during parsing, see below in
-                               msgfmt_frob_new_message.  */
-                            0, 0,
-                            1, check_format_strings, check_header,
-                            check_compatibility,
-                            check_accelerators, accelerator_char);
-
-      /* Exit with status 1 on any error.  */
-      if (nerrors > 0)
-        {
-          error (0, 0,
-                 ngettext ("found %d fatal error", "found %d fatal errors",
-                           nerrors),
-                 nerrors);
-          retval = EXIT_FAILURE;
-          goto out;
-        }
-
-      /* Convert the messages to Unicode.  */
-      iconv_message_list (messages[i], NULL, po_charset_utf8, NULL);
+      msgfmt_operand_list_destroy (&operands);
+      return 1;
     }
 
   /* Write the messages into .desktop file.  */
-  if (msgdomain_write_desktop_bulk (languages,
-                                    messages,
-                                    template_file_name,
-                                    keywords,
-                                    file_name))
+  status = msgdomain_write_desktop_bulk (&operands,
+                                         template_file_name,
+                                         keywords,
+                                         file_name);
+
+  msgfmt_operand_list_destroy (&operands);
+
+  return status;
+}
+
+/* Helper function to support 'bulk' operation mode of --xml.
+   This reads all .po files in DIRECTORY and merges them into an
+   XML file FILE_NAME.  Currently it does not support some
+   options available in 'iterative' mode, such as --statistics.  */
+static int
+msgfmt_xml_bulk (const char *directory,
+                 const char *template_file_name,
+                 its_rule_list_ty *its_rules,
+                 const char *file_name)
+{
+  msgfmt_operand_list_ty operands;
+  int nerrors, status;
+
+  msgfmt_operand_list_init (&operands);
+
+  /* Read all .po files.  */
+  nerrors = msgfmt_operand_list_add_from_directory (&operands, directory);
+  if (nerrors > 0)
     {
-      retval = EXIT_FAILURE;
-      goto out;
+      msgfmt_operand_list_destroy (&operands);
+      return 1;
     }
 
- out:
-  dir_list_restore (saved_dir_list);
-  for (i = 0; i < languages->nitems; i++)
-    message_list_free (messages[i], 0);
-  free (messages);
-  string_list_free (languages);
+  /* Write the messages into .xml file.  */
+  status = msgdomain_write_xml_bulk (&operands,
+                                     template_file_name,
+                                     its_rules,
+                                     file_name);
 
-  return retval;
+  msgfmt_operand_list_destroy (&operands);
+
+  return status;
 }
