@@ -1,6 +1,5 @@
 /* Message list charset and locale charset handling.
-   Copyright (C) 2001-2003, 2005-2009, 2015 Free Software Foundation,
-   Inc.
+   Copyright (C) 2001-2003, 2005-2009, 2019 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -14,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #ifdef HAVE_CONFIG_H
@@ -67,8 +66,7 @@ conversion_error (const struct conversion_context* context)
                           context->from_filename, context->from_code));
   else
     po_xerror (PO_SEVERITY_FATAL_ERROR, context->message, NULL, 0, 0, false,
-               xasprintf (_("\
-%s: error while converting from \"%s\" encoding to \"%s\" encoding"),
+               xasprintf (_("%s: error while converting from \"%s\" encoding to \"%s\" encoding"),
                           context->from_filename, context->from_code,
                           context->to_code));
   /* NOTREACHED */
@@ -245,8 +243,8 @@ iconv_message_list_internal (message_list_ty *mlp,
                           canon_charset = po_charset_ascii;
                         else
                           po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0,
-                                     false, xasprintf (_("\
-present charset \"%s\" is not a portable encoding name"),
+                                     false,
+                                     xasprintf (_("present charset \"%s\" is not a portable encoding name"),
                                                 charset));
                       }
                   }
@@ -257,8 +255,7 @@ present charset \"%s\" is not a portable encoding name"),
                     else if (canon_from_code != canon_charset)
                       po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0,  0,
                                  false,
-                                 xasprintf (_("\
-two different charsets \"%s\" and \"%s\" in input file"),
+                                 xasprintf (_("two different charsets \"%s\" and \"%s\" in input file"),
                                             canon_from_code, canon_charset));
                   }
                 freea (charset);
@@ -288,8 +285,7 @@ two different charsets \"%s\" and \"%s\" in input file"),
         canon_from_code = po_charset_ascii;
       else
         po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
-                   _("\
-input file doesn't contain a header entry with a charset specification"));
+                   _("input file doesn't contain a header entry with a charset specification"));
     }
 
   msgids_changed = false;
@@ -303,9 +299,7 @@ input file doesn't contain a header entry with a charset specification"));
 
       if (iconveh_open (canon_to_code, canon_from_code, &cd) < 0)
         po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
-                   xasprintf (_("\
-Cannot convert from \"%s\" to \"%s\". %s relies on iconv(), \
-and iconv() does not support this conversion."),
+                   xasprintf (_("Cannot convert from \"%s\" to \"%s\". %s relies on iconv(), and iconv() does not support this conversion."),
                               canon_from_code, canon_to_code,
                               basename (program_name)));
 
@@ -333,15 +327,11 @@ and iconv() does not support this conversion."),
       if (msgids_changed)
         if (message_list_msgids_changed (mlp))
           po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
-                     xasprintf (_("\
-Conversion from \"%s\" to \"%s\" introduces duplicates: \
-some different msgids become equal."),
+                     xasprintf (_("Conversion from \"%s\" to \"%s\" introduces duplicates: some different msgids become equal."),
                                 canon_from_code, canon_to_code));
 #else
           po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
-                     xasprintf (_("\
-Cannot convert from \"%s\" to \"%s\". %s relies on iconv(). \
-This version was built without iconv()."),
+                     xasprintf (_("Cannot convert from \"%s\" to \"%s\". %s relies on iconv(). This version was built without iconv()."),
                                 canon_from_code, canon_to_code,
                                 basename (program_name)));
 #endif
@@ -373,8 +363,7 @@ iconv_msgdomain_list (msgdomain_list_ty *mdlp,
   canon_to_code = po_charset_canonicalize (to_code);
   if (canon_to_code == NULL)
     po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
-               xasprintf (_("\
-target charset \"%s\" is not a portable encoding name."),
+               xasprintf (_("target charset \"%s\" is not a portable encoding name."),
                           to_code));
 
   for (k = 0; k < mdlp->nitems; k++)
