@@ -1,6 +1,5 @@
 /* xgettext JavaScript backend.
-   Copyright (C) 2002-2003, 2005-2009, 2013, 2015 Free Software
-   Foundation, Inc.
+   Copyright (C) 2002-2003, 2005-2009, 2013-2014, 2018-2019 Free Software Foundation, Inc.
 
    This file was written by Andreas Stricker <andy@knitter.ch>, 2010
    It's based on x-python from Bruno Haible.
@@ -16,7 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -33,7 +32,15 @@
 #include <string.h>
 
 #include "message.h"
+#include "rc-str-list.h"
 #include "xgettext.h"
+#include "xg-pos.h"
+#include "xg-encoding.h"
+#include "xg-mixed-string.h"
+#include "xg-arglist-context.h"
+#include "xg-arglist-callshape.h"
+#include "xg-arglist-parser.h"
+#include "xg-message.h"
 #include "error.h"
 #include "error-progname.h"
 #include "progname.h"
@@ -55,7 +62,7 @@
 
 /* The JavaScript aka ECMA-Script syntax is defined in ECMA-262
    specification:
-   http://www.ecma-international.org/publications/standards/Ecma-262.htm */
+   https://www.ecma-international.org/publications/standards/Ecma-262.htm */
 
 /* ====================== Keyword set customization.  ====================== */
 
@@ -136,13 +143,6 @@ init_flag_table_javascript ()
 
 /* ======================== Reading of characters.  ======================== */
 
-/* Real filename, used in error messages about the input file.  */
-static const char *real_file_name;
-
-/* Logical filename and line number, used to label the extracted messages.  */
-static char *logical_file_name;
-static int line_number;
-
 /* The input file stream.  */
 static FILE *fp;
 
@@ -228,8 +228,7 @@ phase2_getc ()
                                       non_ascii_error_message (lexical_context,
                                                                real_file_name,
                                                                line_number),
-                                      _("\
-Please specify the source encoding through --from-code\n")));
+                                      _("Please specify the source encoding through --from-code\n")));
           exit (EXIT_FAILURE);
         }
       return c;
@@ -447,76 +446,7 @@ phase2_ungetc (int c)
 
 /* ========================= Accumulating strings.  ======================== */
 
-/* A string buffer type that allows appending Unicode characters.
-   Returns the entire string in UTF-8 encoding.  */
-
-struct unicode_string_buffer
-{
-  /* The part of the string that has already been converted to UTF-8.  */
-  char *utf8_buffer;
-  size_t utf8_buflen;
-  size_t utf8_allocated;
-};
-
-/* Initialize a 'struct unicode_string_buffer' to empty.  */
-static inline void
-init_unicode_string_buffer (struct unicode_string_buffer *bp)
-{
-  bp->utf8_buffer = NULL;
-  bp->utf8_buflen = 0;
-  bp->utf8_allocated = 0;
-}
-
-/* Auxiliary function: Ensure count more bytes are available in bp->utf8.  */
-static inline void
-unicode_string_buffer_append_unicode_grow (struct unicode_string_buffer *bp,
-                                           size_t count)
-{
-  if (bp->utf8_buflen + count > bp->utf8_allocated)
-    {
-      size_t new_allocated = 2 * bp->utf8_allocated + 10;
-      if (new_allocated < bp->utf8_buflen + count)
-        new_allocated = bp->utf8_buflen + count;
-      bp->utf8_allocated = new_allocated;
-      bp->utf8_buffer = xrealloc (bp->utf8_buffer, new_allocated);
-    }
-}
-
-/* Auxiliary function: Append a Unicode character to bp->utf8.
-   uc must be < 0x110000.  */
-static inline void
-unicode_string_buffer_append_unicode (struct unicode_string_buffer *bp,
-                                      unsigned int uc)
-{
-  unsigned char utf8buf[6];
-  int count = u8_uctomb (utf8buf, uc, 6);
-
-  if (count < 0)
-    /* The caller should have ensured that uc is not out-of-range.  */
-    abort ();
-
-  unicode_string_buffer_append_unicode_grow (bp, count);
-  memcpy (bp->utf8_buffer + bp->utf8_buflen, utf8buf, count);
-  bp->utf8_buflen += count;
-}
-
-/* Return the string buffer's contents.  */
-static char *
-unicode_string_buffer_result (struct unicode_string_buffer *bp)
-{
-  /* NUL-terminate it.  */
-  unicode_string_buffer_append_unicode_grow (bp, 1);
-  bp->utf8_buffer[bp->utf8_buflen] = '\0';
-  /* Return it.  */
-  return bp->utf8_buffer;
-}
-
-/* Free the memory pointed to by a 'struct unicode_string_buffer'.  */
-static inline void
-free_unicode_string_buffer (struct unicode_string_buffer *bp)
-{
-  free (bp->utf8_buffer);
-}
+/* See xg-mixed-string.h for the API.  */
 
 
 /* ======================== Accumulating comments.  ======================== */
@@ -524,31 +454,32 @@ free_unicode_string_buffer (struct unicode_string_buffer *bp)
 
 /* Accumulating a single comment line.  */
 
-static struct unicode_string_buffer comment_buffer;
+static struct mixed_string_buffer comment_buffer;
 
 static inline void
 comment_start ()
 {
-  lexical_context = lc_comment;
-  comment_buffer.utf8_buflen = 0;
+  mixed_string_buffer_init (&comment_buffer, lc_comment,
+                            logical_file_name, line_number);
 }
 
 static inline bool
 comment_at_start ()
 {
-  return (comment_buffer.utf8_buflen == 0);
+  return mixed_string_buffer_is_empty (&comment_buffer);
 }
 
 static inline void
 comment_add (int c)
 {
-  unicode_string_buffer_append_unicode (&comment_buffer, c);
+  mixed_string_buffer_append_unicode (&comment_buffer, c);
 }
 
 static inline const char *
 comment_line_end (size_t chars_to_remove)
 {
-  char *buffer = unicode_string_buffer_result (&comment_buffer);
+  char *buffer =
+    mixed_string_contents_free1 (mixed_string_buffer_result (&comment_buffer));
   size_t buflen = strlen (buffer) - chars_to_remove;
 
   while (buflen >= 1
@@ -702,6 +633,7 @@ phase3_ungetc (int c)
 /* Return value of phase7_getuc when EOF is reached.  */
 #define P7_EOF (-1)
 #define P7_STRING_END (-2)
+#define P7_TEMPLATE_START_OF_EXPRESSION (-3) /* ${ */
 
 /* Convert an UTF-16 or UTF-32 code point to a return value that can be
    distinguished from a single-byte return value.  */
@@ -732,6 +664,10 @@ enum token_type_ty
   token_type_operator,          /* - * / % . < > = ~ ! | & ? : ^ */
   token_type_equal,             /* = */
   token_type_string,            /* "abc", 'abc' */
+  token_type_template,          /* `abc` */
+  token_type_ltemplate,         /* left part of template: `abc${ */
+  token_type_mtemplate,         /* middle part of template: }abc${ */
+  token_type_rtemplate,         /* right part of template: }abc` */
   token_type_keyword,           /* return, else */
   token_type_symbol,            /* symbol, number */
   token_type_other              /* misc. operator */
@@ -742,9 +678,9 @@ typedef struct token_ty token_ty;
 struct token_ty
 {
   token_type_ty type;
-  char *string;         /* for token_type_string, token_type_symbol,
-                           token_type_keyword */
-  refcounted_string_list_ty *comment;   /* for token_type_string */
+  char *string;                  /* for token_type_symbol, token_type_keyword */
+  mixed_string_ty *mixed_string;        /* for token_type_string, token_type_template */
+  refcounted_string_list_ty *comment;   /* for token_type_string, token_type_template */
   int line_number;
 };
 
@@ -753,15 +689,18 @@ struct token_ty
 static inline void
 free_token (token_ty *tp)
 {
-  if (tp->type == token_type_string || tp->type == token_type_symbol)
+  if (tp->type == token_type_symbol || tp->type == token_type_keyword)
     free (tp->string);
-  if (tp->type == token_type_string)
-    drop_reference (tp->comment);
+  if (tp->type == token_type_string || tp->type == token_type_template)
+    {
+      mixed_string_free (tp->mixed_string);
+      drop_reference (tp->comment);
+    }
 }
 
 
 /* JavaScript provides strings with either double or single quotes:
-     "abc" or 'abc'
+     "abc" or 'abc' or `abc`
    Both may contain special sequences after a backslash:
      \', \", \\, \b, \f, \n, \r, \t, \v
    Special characters can be entered using hexadecimal escape
@@ -789,14 +728,39 @@ phase7_getuc (int quote_char)
       if (c == quote_char)
         return P7_STRING_END;
 
+      if (c == '$' && quote_char == '`')
+        {
+          int c1 = phase2_getc ();
+
+          if (c1 == '{')
+            return P7_TEMPLATE_START_OF_EXPRESSION;
+          phase2_ungetc (c1);
+        }
+
       if (c == '\n')
         {
-          phase2_ungetc (c);
-          error_with_progname = false;
-          error (0, 0, _("%s:%d: warning: unterminated string"),
-                 logical_file_name, line_number);
-          error_with_progname = true;
-          return P7_STRING_END;
+          if (quote_char == '`')
+            return UNICODE ('\n');
+          else
+            {
+              phase2_ungetc (c);
+              error_with_progname = false;
+              error (0, 0, _("%s:%d: warning: unterminated string"),
+                     logical_file_name, line_number);
+              error_with_progname = true;
+              return P7_STRING_END;
+            }
+        }
+
+      if (c == '\r' && quote_char == '`')
+        {
+          /* Line terminators inside template literals are normalized to \n,
+             says <http://exploringjs.com/es6/ch_template-literals.html>.  */
+          int c1 = phase2_getc ();
+
+          if (c1 == '\n')
+            return UNICODE ('\n');
+          phase2_ungetc (c1);
         }
 
       if (c != '\\')
@@ -931,7 +895,7 @@ static int phase5_pushback_length;
 static token_type_ty last_token_type = token_type_other;
 
 static void
-phase5_scan_regexp ()
+phase5_scan_regexp (void)
 {
     int c;
 
@@ -965,8 +929,15 @@ phase5_scan_regexp ()
       phase2_ungetc (c);
 }
 
-static int xml_element_depth = 0;
-static bool inside_embedded_js_in_xml = false;
+/* Number of open '{' tokens.  */
+static int brace_depth;
+
+/* Number of open template literals `...${  */
+static int template_literal_depth;
+
+/* Number of open XML elements.  */
+static int xml_element_depth;
+static bool inside_embedded_js_in_xml;
 
 static bool
 phase5_scan_xml_markup (token_ty *tp)
@@ -1189,43 +1160,88 @@ phase5_get (token_ty *tp)
             return;
           }
 
-        /* Strings.  */
+        case '"': case '\'':
+          /* Strings.  */
           {
-            struct mixed_string_buffer *bp;
             int quote_char;
+            struct mixed_string_buffer msb;
 
-            case '"': case '\'':
-              quote_char = c;
-              lexical_context = lc_string;
-              /* Start accumulating the string.  */
-              bp = mixed_string_buffer_alloc (lexical_context,
-                                              logical_file_name,
-                                              line_number);
-              for (;;)
-                {
-                  int uc = phase7_getuc (quote_char);
+            quote_char = c;
+            lexical_context = lc_string;
+            /* Start accumulating the string.  */
+            mixed_string_buffer_init (&msb, lexical_context,
+                                      logical_file_name, line_number);
+            for (;;)
+              {
+                int uc = phase7_getuc (quote_char);
 
-                  /* Keep line_number in sync.  */
-                  bp->line_number = line_number;
+                /* Keep line_number in sync.  */
+                msb.line_number = line_number;
 
-                  if (uc == P7_EOF || uc == P7_STRING_END)
+                if (uc == P7_EOF || uc == P7_STRING_END)
+                  break;
+
+                if (IS_UNICODE (uc))
+                  {
+                    assert (UNICODE_VALUE (uc) >= 0
+                            && UNICODE_VALUE (uc) < 0x110000);
+                    mixed_string_buffer_append_unicode (&msb,
+                                                        UNICODE_VALUE (uc));
+                  }
+                else
+                  mixed_string_buffer_append_char (&msb, uc);
+              }
+            tp->mixed_string = mixed_string_buffer_result (&msb);
+            tp->comment = add_reference (savable_comment);
+            lexical_context = lc_outside;
+            tp->type = last_token_type = token_type_string;
+            return;
+          }
+
+        case '`':
+          /* Template literals.  */
+          {
+            struct mixed_string_buffer msb;
+
+            lexical_context = lc_string;
+            /* Start accumulating the string.  */
+            mixed_string_buffer_init (&msb, lexical_context,
+                                      logical_file_name, line_number);
+            for (;;)
+              {
+                int uc = phase7_getuc ('`');
+
+                /* Keep line_number in sync.  */
+                msb.line_number = line_number;
+
+                if (uc == P7_EOF || uc == P7_STRING_END)
+                  {
+                    tp->mixed_string = mixed_string_buffer_result (&msb);
+                    tp->comment = add_reference (savable_comment);
+                    tp->type = last_token_type = token_type_template;
                     break;
+                  }
 
-                  if (IS_UNICODE (uc))
-                    {
-                      assert (UNICODE_VALUE (uc) >= 0
-                              && UNICODE_VALUE (uc) < 0x110000);
-                      mixed_string_buffer_append_unicode (bp,
-                                                          UNICODE_VALUE (uc));
-                    }
-                  else
-                    mixed_string_buffer_append_char (bp, uc);
-                }
-              tp->string = mixed_string_buffer_done (bp);
-              tp->comment = add_reference (savable_comment);
-              lexical_context = lc_outside;
-              tp->type = last_token_type = token_type_string;
-              return;
+                if (uc == P7_TEMPLATE_START_OF_EXPRESSION)
+                  {
+                    mixed_string_buffer_destroy (&msb);
+                    tp->type = last_token_type = token_type_ltemplate;
+                    template_literal_depth++;
+                    break;
+                  }
+
+                if (IS_UNICODE (uc))
+                  {
+                    assert (UNICODE_VALUE (uc) >= 0
+                            && UNICODE_VALUE (uc) < 0x110000);
+                    mixed_string_buffer_append_unicode (&msb,
+                                                        UNICODE_VALUE (uc));
+                  }
+                else
+                  mixed_string_buffer_append_char (&msb, uc);
+              }
+            lexical_context = lc_outside;
+            return;
           }
 
         case '+':
@@ -1332,7 +1348,7 @@ phase5_get (token_ty *tp)
             tp->type = last_token_type = token_type_operator;
           else
             {
-              phase5_scan_regexp (tp);
+              phase5_scan_regexp ();
               tp->type = last_token_type = token_type_regexp;
             }
           return;
@@ -1340,12 +1356,38 @@ phase5_get (token_ty *tp)
         case '{':
           if (xml_element_depth > 0 && !inside_embedded_js_in_xml)
             inside_embedded_js_in_xml = true;
+          else
+            brace_depth++;
           tp->type = last_token_type = token_type_other;
           return;
 
         case '}':
           if (xml_element_depth > 0 && inside_embedded_js_in_xml)
             inside_embedded_js_in_xml = false;
+          else if (brace_depth > 0)
+            brace_depth--;
+          else if (template_literal_depth > 0)
+            {
+              /* Middle or right part of template literal.  */
+              for (;;)
+                {
+                  int uc = phase7_getuc ('`');
+
+                  if (uc == P7_EOF || uc == P7_STRING_END)
+                    {
+                      tp->type = last_token_type = token_type_rtemplate;
+                      template_literal_depth--;
+                      break;
+                    }
+
+                  if (uc == P7_TEMPLATE_START_OF_EXPRESSION)
+                    {
+                      tp->type = last_token_type = token_type_mtemplate;
+                      break;
+                    }
+                }
+              return;
+            }
           tp->type = last_token_type = token_type_other;
           return;
 
@@ -1392,16 +1434,16 @@ phase5_unget (token_ty *tp)
 }
 
 
-/* String concatenation with '+'.  */
+/* String concatenation with '+'.
+   Handling of tagged template literals.  */
 
 static void
 x_javascript_lex (token_ty *tp)
 {
   phase5_get (tp);
-  if (tp->type == token_type_string)
+  if (tp->type == token_type_string || tp->type == token_type_template)
     {
-      char *sum = tp->string;
-      size_t sum_len = strlen (sum);
+      mixed_string_ty *sum = tp->mixed_string;
 
       for (;;)
         {
@@ -1413,14 +1455,10 @@ x_javascript_lex (token_ty *tp)
               token_ty token3;
 
               phase5_get (&token3);
-              if (token3.type == token_type_string)
+              if (token3.type == token_type_string
+                  || token3.type == token_type_template)
                 {
-                  char *addend = token3.string;
-                  size_t addend_len = strlen (addend);
-
-                  sum = (char *) xrealloc (sum, sum_len + addend_len + 1);
-                  memcpy (sum + sum_len, addend, addend_len + 1);
-                  sum_len += addend_len;
+                  sum = mixed_string_concat_free1 (sum, token3.mixed_string);
 
                   free_token (&token3);
                   free_token (&token2);
@@ -1431,7 +1469,25 @@ x_javascript_lex (token_ty *tp)
           phase5_unget (&token2);
           break;
         }
-      tp->string = sum;
+      tp->mixed_string = sum;
+    }
+  else if (tp->type == token_type_symbol)
+    {
+      token_ty token2;
+
+      phase5_get (&token2);
+      if (token2.type == token_type_template)
+        {
+          /* The value of
+               tag `abc`
+             is the value of the function call
+               tag (["abc"])
+             We don't know anything about this value.  Therefore, don't
+             let the extractor see this template literal.  */
+          free_token (&token2);
+        }
+      else
+        phase5_unget (&token2);
     }
 }
 
@@ -1522,9 +1578,7 @@ extract_balanced (message_list_ty *mlp,
                                 arglist_parser_alloc (mlp,
                                                       state ? next_shapes : NULL)))
             {
-              xgettext_current_source_encoding = po_charset_utf8;
               arglist_parser_done (argparser, arg);
-              xgettext_current_source_encoding = xgettext_current_file_source_encoding;
               return true;
             }
           next_context_iter = null_context_list_iterator;
@@ -1534,9 +1588,7 @@ extract_balanced (message_list_ty *mlp,
         case token_type_rparen:
           if (delim == token_type_rparen || delim == token_type_eof)
             {
-              xgettext_current_source_encoding = po_charset_utf8;
               arglist_parser_done (argparser, arg);
-              xgettext_current_source_encoding = xgettext_current_file_source_encoding;
               return false;
             }
           next_context_iter = null_context_list_iterator;
@@ -1558,9 +1610,7 @@ extract_balanced (message_list_ty *mlp,
                                 null_context, null_context_list_iterator,
                                 arglist_parser_alloc (mlp, NULL)))
             {
-              xgettext_current_source_encoding = po_charset_utf8;
               arglist_parser_done (argparser, arg);
-              xgettext_current_source_encoding = xgettext_current_file_source_encoding;
               return true;
             }
           next_context_iter = null_context_list_iterator;
@@ -1570,9 +1620,7 @@ extract_balanced (message_list_ty *mlp,
         case token_type_rbracket:
           if (delim == token_type_rbracket || delim == token_type_eof)
             {
-              xgettext_current_source_encoding = po_charset_utf8;
               arglist_parser_done (argparser, arg);
-              xgettext_current_source_encoding = xgettext_current_file_source_encoding;
               return false;
             }
           next_context_iter = null_context_list_iterator;
@@ -1580,21 +1628,25 @@ extract_balanced (message_list_ty *mlp,
           continue;
 
         case token_type_string:
+        case token_type_template:
           {
             lex_pos_ty pos;
+
             pos.file_name = logical_file_name;
             pos.line_number = token.line_number;
 
-            xgettext_current_source_encoding = po_charset_utf8;
             if (extract_all)
-              remember_a_message (mlp, NULL, token.string, inner_context,
-                                  &pos, NULL, token.comment);
+              {
+                char *string = mixed_string_contents (token.mixed_string);
+                mixed_string_free (token.mixed_string);
+                remember_a_message (mlp, NULL, string, true, inner_context,
+                                    &pos, NULL, token.comment, true);
+              }
             else
-              arglist_parser_remember (argparser, arg, token.string,
+              arglist_parser_remember (argparser, arg, token.mixed_string,
                                        inner_context,
                                        pos.file_name, pos.line_number,
-                                       token.comment);
-            xgettext_current_source_encoding = xgettext_current_file_source_encoding;
+                                       token.comment, true);
           }
           drop_reference (token.comment);
           next_context_iter = null_context_list_iterator;
@@ -1602,11 +1654,12 @@ extract_balanced (message_list_ty *mlp,
           continue;
 
         case token_type_eof:
-          xgettext_current_source_encoding = po_charset_utf8;
           arglist_parser_done (argparser, arg);
-          xgettext_current_source_encoding = xgettext_current_file_source_encoding;
           return true;
 
+        case token_type_ltemplate:
+        case token_type_mtemplate:
+        case token_type_rtemplate:
         case token_type_keyword:
         case token_type_plus:
         case token_type_regexp:
@@ -1642,7 +1695,10 @@ extract_javascript (FILE *f,
   last_comment_line = -1;
   last_non_comment_line = -1;
 
+  brace_depth = 0;
+  template_literal_depth = 0;
   xml_element_depth = 0;
+  inside_embedded_js_in_xml = false;
 
   xgettext_current_file_source_encoding = xgettext_global_source_encoding;
 #if HAVE_ICONV
